/*+		T D T

 * Module name:
     tdt

 * Function:
     {short description of module function}

 * Description:
     {more details description}

 * Language:
      C

 * Support: {author}, AAO

 *-

 * "@(#) $Id: ACMM:acmmTemplates/drama/tdt.c,v 1.24 12-Aug-2021 17:00:24+10 tjf $"

 * History:
     {dd-mmm-yyyy} - {initials} - Original version

 *  Copyright (c) Macquarie University, {year}.
    Not to be used for commercial purposes without permission.
    No warranty of any form unless subject to a specific agreement.

 */
/*
 * RCS id
 */
static const char *rcsId="@(#) $Id: ACMM:acmmTemplates/drama/tdt.c,v 1.24 12-Aug-2021 17:00:24+10 tjf $";
static void *use_rcsId = (0 ? (void *)(&use_rcsId) : (void *) &rcsId);
/*
 *  Include files
 */
#include "tdt.h"                        /* Out include file */
#include "DitsSys.h"                    /* For PutActionHandlers        */
#include "DitsMsgOut.h"                 /* For MsgOut                   */
#include "arg.h"                        /* For ARG_ macros              */
#include "Sdp.h"                        /* Sdp routines                 */
#include "Ers.h"                        /* Ers routines                 */
#include "Git.h"                        /* Git routines                 */
#include "Git_Err.h"                    /* GIT__ codes                  */

#include "tdt_err.h"
#include "mess.h"
#include "tdt_err_msgt.h"
/* 
 * Constant definitions
 */

/*
 * Macro function definitions
 */

/*
 * type definitions
 */

/*
 *  This typedef defines a structure used to store any information specific
 *  to this library on a per-program basis (would traditionally be
 *  static/global objects under Unix)
 */
typedef struct {
    int initialised;
} TdtDetailsType;


/*
 * Global variables and functions imported (but not declared in include files)
 */

/*
 *  TdtVerson and TdtDate are defined in the module tdtversion.c
 */
extern const char * const TdtVersion;
extern const char * const TdtDate;


/*
 * Global variables exported (definitions and initialization of globals
 * declared in the associated include file)
 */

/*
 * Variables private to this module (static variables)
 *
 *
 *  Action defintions, first the prototypes and then the action map array
 */
static void TdtInitialise(StatusType *status);
static void TdtFilter1   (StatusType *status);
static void TdtFilter2   (StatusType *status);

static DitsActionMapType TdtMap[] = {
    /*
     * Override the GIT INITIALISE action.
     */
    {TdtInitialise, 0, 0, "INITIALISE"},
    /*
     * Add our own task specific actions.
     */
    {TdtFilter1,    0, 0, "FILTER1" },
    {TdtFilter2,    0, 0, "FILTER2" }
         };
/*
 *  Now the parameter definitions.  First define some values we need addresses
 *  of and the and paramater definition array.
 */
static int one = 1;
static float two = 2.2;
static unsigned int three = 3;

static SdpParDefType TdtParams[] = {
/*
 * Override these Git parameters
 */
    { "ENQ_DEV_DESCR",   "{Template Drama Task}", ARG_STRING },
    { "ENQ_VER_NUM",     "",                    ARG_STRING },
    { "ENQ_VER_DATE",    "",                    ARG_STRING },
/*
 * Task specific parameters
 */
    { "PARAM1", &one,       SDS_INT    },
    { "PARAM2", "hi there", ARG_STRING },
    { "PARAM3", &two,       SDS_FLOAT  },
    { "PARAM4", &three,     SDS_UINT   }};

/*
 *  Internal Function name:
      TdtMyDetails

 *  Description:
       Convenience function which combines a look up using GitTpiGet with
       a check that the library's initialise call has been invoked.
 
       {Example note - this does not have to be static if there i
        is more then on C module in the task which needs this }

 *  History:
     {dd-mmm-yyyy} - {initials} - Original version
     {@change entry@}
 */
static TdtDetailsType *TdtDetails(StatusType *status)
{
    TdtDetailsType *details = 0;

    if (*status != STATUS__OK) return 0;


    GitTpiGet(TDT__FACNUM,
              (void *)&details,
              status);

    if (*status == GIT__TPI_NOTFOUND)
        ErsRep(0,status,"TDT module not activated");
    else if ((*status == STATUS__OK)&&(!details->initialised))
        *status = TDT__NOTINIT;
    return(details);
}



/*
 *+			T D T A C T I V A T E

 * Function name:
      TdtActivate

 * Function:
       Activate {this module's} task action handlers.

 * Description:
      Actions handlers are put using DitsPutActionHandlers and the 
      parameters  are created using SdpCreate.   The structure used
      to store internal task details is created and stored using GitTpi.

 * Language:
      C

 * Call:
      (Void) = TdtActivate (parsysid, status)

 *  Parameters:   (">" input, "!" modified, "W" workspace, "<" output)
      (>) parsysid      (SdsIdType) The parameter system id returned by a 
                            call to SdpInit
      (!) status        (StatusType *) Modified status. 


 * Include files: Tdt.h

 * See Also: {references}

 * External functions used:
      {external functions}

 * External values used:
      {external variables referenced}

 * Prior requirements:
      DitsAppInit and SdpInit should have been called.  Should not be called
      from a Dits action handler routine.
      {List any other prior requirements}

 * Support: {function author}, AAO

 *-

 * History:
     {dd-mmm-yyyy} - {initials} - Original version
 */
extern void TdtActivate (SdsIdType parsysid, StatusType *status)
{
    TdtDetailsType *details;
/*
 *  Add the message facility codes which may be generated by this program.
 */
    MessPutFacility(&MessFac_TDT);
/*
 *  Add the actions and parameters
 */
    DitsPutActionHandlers(DitsNumber(TdtMap),TdtMap,status);
    SdpCreate(parsysid, DitsNumber(TdtParams),TdtParams,status);

/*
 *  Put the version number and date parameter values
 */
    SdpPutString ("ENQ_VER_NUM",TdtVersion,status);
    SdpPutString ("ENQ_VER_DATE",TdtDate,status);

/*
 *  Create the details structure.
 */
    details = (TdtDetailsType *)malloc(sizeof(TdtDetailsType));
    if (details == 0)
    {
        *status = TDT__MALLOCERR;
        return;
    }
/*
 *  Store the details using GitTpi
 */
    GitTpiPut(TDT__FACNUM,details,status);
/*
 *  Note we have not been initliased.
 */
    details->initialised = 0;
}


/*
 *+			T D T D E A C T I V A T E

 * Function name:
      TdtDeActivate

 * Function:
       DeActivate {this module's} task action handlers.

 * Description:

 * Language:
      C

 * Call:
      (Void) = TdtDeActivate (parsysid, status)

 *  Parameters:   (">" input, "!" modified, "W" workspace, "<" output)
      (!) status        (StatusType *) Modified status. 


 * Include files: Tdt.h

 * See Also: {references}

 * External functions used:
      {external functions}

 * External values used:
      {external variables referenced}

 * Prior requirements:

 * Support: {function author}, AAO

 *-

 * History:
     {dd-mmm-yyyy} - {initials} - Original version
 */
extern void TdtDeActivate (StatusType *status)
{
    if (*status != STATUS__OK) return; 
}

/*
 *  Internal Function name:
      TdtInitialise

 *  Description:
      Initialise the TDT task.

 *  History:
     {dd-mmm-yyyy} - {initials} - Original version
     {@change entry@}
 */

static void TdtInitialise(StatusType *status)
{
    TdtDetailsType *details = 0;

    ErsPush();
    details = TdtDetails(status);

    if (*status == TDT__NOTINIT)
    {
        ErsAnnul(status);
        details->initialised = 1;
    }
    else if (*status == STATUS__OK)
    {
        /* Already intialised, may or may not be an error */
    }
    ErsPop();


}




/*
 *  Internal Function name:
      TdtFilter1

 *  Description:
      Tdt filter number 1 command

 *  History:
     {dd-mmm-yyyy} - {initials} - Original version
     {@change entry@}
 */

static void TdtFilter1(StatusType *status)
{
    /*
     * At the moment, the main effect of this is to check that
     * the task has been initialised - hence the resultant
     * variable is unused and is marked as such.   The DUNUSED
     * flag should be removed when the details structure is accessed.
     */
    TdtDetailsType *details DUNUSED = TdtDetails(status);

    if (*status != STATUS__OK) return;

    MsgOut(status,"Moving filter %d",1);
}

/*
 *  Internal Function, name:
      TdtFilter2

 *  Description:
      Tdt filter number 1 command

 *  History:
     {dd-mmm-yyyy} - {initials} - Original version
      {@change entry@}
 */

static void TdtFilter2(StatusType *status)
{
    TdtDetailsType *details DUNUSED = TdtDetails(status);

    if (*status != STATUS__OK) return;

    MsgOut(status,"Moving filter %d",2);
}
